from exchangelib import Credentials, Account

import logging

logger = logging.getLogger(__name__)


class MailerException(Exception):
    ...


class MailReceiver:
    def __init__(self, username: str, password: str, primary_smtp_address: str):
        self.credentials = Credentials(username, password)
        self.account = Account(primary_smtp_address, credentials=self.credentials, autodiscover=True)

    def get_last_inbox_by_params(self, params: str):
        """
        Get last email
        :param params: str
        :return:
        """
        for item in self.account.inbox.all().order_by("-datetime_received")[:1]:
            if params in item.subject:
                return item.body
            else:
                raise MailerException("No email!")



import json
import logging
from time import sleep
import sys
from datetime import datetime
import hashlib
from typing import Union


logger = logging.getLogger(__name__)


def process_browser_log_entry(entry):
    """Принимаем даннаые из performance и возвращаем json"""
    response = json.loads(entry['message'])['message']
    return response


def check_requests(driver):
    """
    Функция смотрит в DevTools Chrome в раздел Network.
    Если у запроса ответ не 200 или 204, кладет в лог урл и статус
    """
    statuses = [200, 204, 304]
    for performance_data in driver.get_log('performance'):
        requests_json = process_browser_log_entry(performance_data)
        if requests_json["method"] == "Network.responseReceived":
            if requests_json["params"]["response"]["status"] in statuses:
                pass
            else:
                logger.error("Request Url: %s, status - %s",
                             requests_json["params"]["response"]["url"],
                             requests_json["params"]["response"]["status"])


def notifies_needed():
    """Функция, проверяющая аргументы на наличие --send-notifies (активация отправки уведомлений в ТГ)"""
    if '--send-notifies' in sys.argv:
        logger.info("Notifies are enabled!")
        return True
    else:
        logger.info("Notifies are disabled!")
        return False


def screencast_needed():
    """Функция, проверяющая аргументы на наличие --screencast (запись скринкаста)"""
    if '--screencast' in sys.argv:
        logger.info("Screencast is enabled!")
        return True
    else:
        logger.info("Screencast is disabled!")
        return False


def get_response_data(driver, url: str, wait: int = None):
    """Функция для логгирования ответа от эндпойнта, url которого передаем аргументом"""
    logger.info("Try to get response data for endpoint %s", url)
    if wait:
        sleep(wait)
    browser_log = driver.get_log('performance')
    events = [process_browser_log_entry(entry) for entry in browser_log]
    events = [event for event in events if 'Network.response' in event['method']]
    tmp_event_list = []
    event_list = []
    for event in events:
        try:
            event_type = event['params']['type']
        except Exception:
            event_type = 'None'
        if event_type == 'XHR':
            tmp_event_list.append(event)
    for event in tmp_event_list:
        try:
            event_url = event['params']['response']['url']
        except Exception:
            event_url = "None"
        if url == event_url:
            event_list.append(event)

    if len(event_list) > 0:
        for i in range(len(event_list)):
            response = driver.execute_cdp_cmd('Network.getResponseBody',
                                              {'requestId': event_list[i]["params"]["requestId"]})
            logger.info("Endpoint's %s response: %s", url, response['body'])
    else:
        logger.error("Responses list is empty")


def get_datetime():
    now = datetime.now()
    logger.info("Fixed datetime: %s", now)
    return now


def get_timediff(start_time: datetime, end_time: datetime):
    time_diff = end_time - start_time
    logger.info("Time diff: %s", time_diff)
    return time_diff


def get_used_memory(driver):
    """Смотрим в консоли, сколько памяти загрузил в себя js"""
    return driver.execute_script("""var mem = window.performance.memory.usedJSHeapSize /  1048576; return mem;""")


def to_hash(data: str, algorithm: Union[str, bytes]):
    """Вычисление хэша данных с указанием алгоритма при помощи библиотеки hashlib."""
    try:
        hash_generator = getattr(hashlib, algorithm)()
    except AttributeError:
        raise AttributeError("Not supported algorithm. Supported only: sha1, sha256, sha512")

    hash_generator.update(data.encode())
    return hash_generator.hexdigest()



import pytest
from OSGRMATT.notifier import NotifySender
from datetime import datetime
from bs4 import BeautifulSoup
import json
from pathlib import Path
import os
import dotenv


def pytest_addoption(parser):
    """Добавление опции отправки уведомлений в ТГ"""
    parser.addoption("--send-notifies", action="store_true", default=False, help="Enable Telegram Notifies")
    parser.addoption("--screencast", action="store_true", default=False, help="Enable Screencast Writing")


def check_notifies(config):
    """Проверка опций на наличие --send-notifies"""
    notifies = config.getoption("--send-notifies")
    if notifies:
        return True
    else:
        return False


@pytest.hookimpl(tryfirst=True)
def pytest_configure(config):
    """Данная функция берет из ini название отчета и меняет его на уникальное"""
    base_dir = Path(__file__).parent.resolve()
    if len(config.invocation_params.args) > 0:
        try:
            file_name = config.invocation_params.args[0].split('/')[1].split('.')[0]
        except IndexError:
            file_name = config.invocation_params.args[0]
    else:
        file_name = 'no_args'
    config.option.htmlpath = (f'{base_dir}/html_reports/'
                              f'report_{file_name}_{datetime.now().strftime("%d-%m-%Y %H-%M-%S")}.html')
    alluredir = getattr(config.option, "allure_report_dir", None)
    if not alluredir:
        setattr(config.option, "allure_report_dir", f"{base_dir}/html_reports/'")


def pytest_unconfigure(config):
    """Данная функция срабатывает после тирдауна,
    забирает путь к отчету и отправляет его в нотифай если тест запущен с опцией --send-notifies"""
    notifies = check_notifies(config)
    if notifies:
        dotenv.load_dotenv()
        tg_token = os.getenv('TOKEN')
        chat_id = os.getenv("CHAT_ID")
        notifier = NotifySender(tg_token, chat_id)
        report_path = config.option.htmlpath
        try:
            report_name = config.invocation_params.args[0].split('/')[1].split('.')[0]
        except IndexError:
            report_name = config.invocation_params.args[0]
        """Parse report file"""
        messages = []
        with open(report_path) as file:
            html = file.read()
        soup = BeautifulSoup(html, 'html.parser')
        jdata = soup.find(id="data-container")
        search_data = jdata.get('data-jsonblob')
        search_jdata = json.loads(search_data)
        keys = list(search_jdata['tests'].keys())
        for key in keys:
            for i in search_jdata['tests'][key]:
                test_id = i["testId"].split('::')
                test_name = f"{test_id[1]}::{test_id[2]}"
                if i["result"] == "Passed":
                    result = "✅ Passed"
                elif i["result"] == "Failed":
                    result = "❌ Failed"
                else:
                    result = "❓️ Status N/A ️"
                messages.append(f'{test_name} - {result}')
        statuses = '\n\n'.join(messages)
        message = f"<b>ClientPortal Test Status</b>\n" \
                  f"{statuses}"
        notifier.send_test_status_message(message)
        """Send html report file"""
        caption = f"<b>ClientPortal HTML Report</b>\n" \
                  f"<i>{report_name}</i>\n\n" \
                  f"=========================="
        notifier.send_report_file(report_path, caption)



import json
from functools import wraps
from time import sleep
import logging

logger = logging.getLogger(__name__)


def work_in_new_tab(func):
    """For work in new tab and close it."""
    @wraps(func)
    def switch(*args, **kwargs):
        current_handle = args[0].driver.current_window_handle
        logger.info("Current handle: %s", current_handle)
        args[0].driver.execute_script("window.open();")
        args[0].driver.switch_to.window(args[0].driver.window_handles[-1])
        sleep(3)

        res = func(*args, **kwargs)

        args[0].driver.execute_script("window.close();")
        args[0].driver.switch_to.window(current_handle)

        return res

    return switch


def fix_used_memory(func):
    """usedJSHeapSize to log"""
    @wraps(func)
    def fix_memory(*args, **kwargs):
        before_usage = args[0].driver.execute_script("""var mem = window.performance.memory.usedJSHeapSize / 
        1048576; return mem;""")
        logger.info("Before method %s runs memory usage: %s Mb", func.__name__, before_usage)

        res = func(*args, **kwargs)

        after_usage = args[0].driver.execute_script("""var mem = window.performance.memory.usedJSHeapSize / 
        1048576; return mem;""")
        logger.info("After method %s runs memory usage: %s Mb", func.__name__, after_usage)

        return res

    return fix_memory


def check_socket_data(func):
    """Get socket data from Network DevTools tab"""
    @wraps(func)
    def check_socket(*args, **kwargs):
        res = func(*args, **kwargs)
        for websocket_data in args[0].driver.get_log('performance'):
            ws_json = json.loads((websocket_data['message']))
            if ws_json["message"]["method"] == "Network.webSocketFrameReceived":
                logger.info("Received socket message: %s", str(ws_json["message"]["params"]["timestamp"]) +
                            ws_json["message"]["params"]["response"]["payloadData"])
            if ws_json["message"]["method"] == "Network.webSocketFrameSent":
                logger.info("Sent socket message: %s", ws_json["message"]["params"]["response"]["payloadData"])

        return res

    return check_socket


def upload_headers(func):
    @wraps(func)
    def work_with_headers(self, *args, **kwargs):
        logger.info("Remove headers 'accept' and 'Content-Type'")
        self._headers.pop('accept')
        self._headers.pop('Content-Type')

        res = func(self, *args, **kwargs)

        logger.info("Return headers 'accept' and 'Content-Type' and delete 'X-File-Size'")
        self._headers.pop('X-File-Size')
        self._headers.pop('X-File-Size-From')
        self._headers['accept'] = 'text/plain'
        self._headers['Content-Type'] = 'application/json-patch+json'

        return res
    return work_with_headers



import shutil
from pathlib import Path
from selenium import webdriver
from selenium.common import SessionNotCreatedException
from selenium.webdriver.chrome.options import Options as Chromeoptions
from selenium.webdriver.remote.file_detector import FileDetector
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager, ChromeType
from time import sleep
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.common.action_chains import ActionChains
from OSGRMATT.utils import get_used_memory

import logging

logger = logging.getLogger(__name__)
logging.getLogger("urllib3.connectionpool").setLevel(logging.ERROR)


def get_download_dir():
    """Возвращает путь к папке для скачивания файлов. Не используется."""
    base_dir = Path(__file__).parent.parent.resolve()
    return base_dir / "tmp/downloads"


class LocalFileDetector(FileDetector):
    """Класс отвечает за скачивание и загрузку локальных файлов при использовании ноды грида. Не используется."""
    def is_available(self):
        pass

    def file_detected(self, file_path):
        download_dir = get_download_dir()
        new_file_path = download_dir / Path(file_path).name
        shutil.copyfile(file_path, str(new_file_path))
        logger.info("File saved to path: %s", new_file_path)
        return True

    def is_local_file(self, file_path):
        pass


class Browser:
    def setup_options(self,
                      arguments: list = None,
                      experimental_options: list = None,
                      capabilities: list = None):
        options = Chromeoptions()
        if arguments:
            for argument in arguments:
                options.add_argument(argument)
        if experimental_options:
            for option in experimental_options:
                for key, value in option.items():
                    options.add_experimental_option(key, value)
        if capabilities:
            for capability in capabilities:
                for key, value in capability.items():
                    options.set_capability(key, value)

        return options

    def setup_driver(self, options: Chromeoptions = None, command_executor: str = None):
        logger.info("Setup driver")
        if options is None:
            options = Chromeoptions()
        if command_executor:
            options.add_argument('--headless')
            options.add_argument('-lang=ru')
            """
            Опция ниже не тестировалась. Подробнее об опции:
            https://www.selenium.dev/documentation/webdriver/drivers/remote_webdriver/#enable-downloads-in-the-grid
            options.enable_downloads = True
            """
            cloud_options = {'build': "build_3",
                             'name': "test_abc3",
                             "browserVersion": "120.0.6099.130"}
            logger.info("Cloud options: %s", cloud_options)
            options.set_capability('cloud:options', cloud_options)
            try:
                driver = webdriver.Remote(command_executor=command_executor,
                                          options=options)
            except SessionNotCreatedException as e:
                logger.error("Create session error: %s", e.msg)
                logger.error("Create session error stacktrace: %s", e.stacktrace)
                raise TypeError("Driver creation error: %s", e.msg)
        else:
            service = Service(ChromeDriverManager(chrome_type=ChromeType.GOOGLE).install())
            driver = webdriver.Chrome(service=service, options=options)

        return driver


class SeleniumLogic:
    def __init__(self, driver):
        self.driver = driver

    def click_element(self, by, value):
        WebDriverWait(self.driver, 30).until(EC.element_to_be_clickable((by, value))).click()

    def click_element_fast(self, by, value):
        WebDriverWait(self.driver, 3).until(EC.element_to_be_clickable((by, value))).click()

    def enter_text(self, by, value, text):
        element = WebDriverWait(self.driver, 30).until(EC.visibility_of_element_located((by, value)))
        element.clear()
        element.send_keys(text)

    def enter_text_by_letter(self, by, value, text):
        element = WebDriverWait(self.driver, 30).until(EC.visibility_of_element_located((by, value)))
        element.clear()
        for i in text:
            element.send_keys(i)
            sleep(0.25)

    def enter_text_in_hidden_input(self, by, value, text):
        """Enter path for upload file"""
        element = WebDriverWait(self.driver, 30).until(EC.presence_of_element_located((by, value)))
        element.send_keys(text)

    def find_element_fast(self, by, value):
        return WebDriverWait(self.driver, 5).until(EC.presence_of_element_located((by, value)))

    def find_element(self, by, value):
        return WebDriverWait(self.driver, 30).until(EC.presence_of_element_located((by, value)))

    def find_element_slow(self, by, value):
        return WebDriverWait(self.driver, 180).until(EC.presence_of_element_located((by, value)))

    def find_elements(self, by, value):
        """find all presence elements with same locator"""
        return WebDriverWait(self.driver, 30).until(EC.presence_of_all_elements_located((by, value)))

    def find_visible_elements(self, by, value):
        """find all visible elements with same locator"""
        return WebDriverWait(self.driver, 30).until(EC.visibility_of_all_elements_located((by, value)))

    def find_element_instantly(self, by, value):
        return self.driver.find_element(by, value)

    def check_visible_element(self, by, value):
        return WebDriverWait(self.driver, 30).until(EC.visibility_of_element_located((by, value)))

    def check_visible_fast(self, by, value):
        return WebDriverWait(self.driver, 1).until(EC.visibility_of_element_located((by, value)))

    def wait_invisibility_of_element(self, by, value):
        try:
            WebDriverWait(self.driver, 30).until(EC.invisibility_of_element_located((by, value)))
        except TimeoutException:
            return 'need more TO'

    def wait_invisibility_of_element_slow(self, by, value):
        try:
            WebDriverWait(self.driver, 180).until(EC.invisibility_of_element_located((by, value)))
        except TimeoutException:
            return 'need more TO'

    def send_keys_to_element(self, by, value, keys):
        element = self.driver.find_element(by, value)
        element.send_keys(keys)

    def send_keys_and_press_enter(self, locator, keys):
        element = self.find_element(*locator)
        element.send_keys(keys)
        element.send_keys(Keys.ENTER)

    def get_alert_info(self, by, value):
        alert = self.find_element(by, value).text
        return alert

    def get_alert_info_slow(self, by, value):
        alert = self.find_element_slow(by, value).text
        return alert

    def press_escape(self):
        ActionChains(self.driver).send_keys(Keys.ESCAPE).perform()

    def click_element_parametrized(self, contains_text: str):
        dialog_xpath = f"//*[contains(text(), '{contains_text}')]"
        self.click_element(By.XPATH, dialog_xpath)

    def get_cookies(self):
        return self.driver.get_cookies()

    def get_bearer(self):
        return self.driver.execute_script("return window.localStorage.getItem('token');")

    def ctrl_f5(self):
        ActionChains(self.driver).key_down(Keys.CONTROL).send_keys(Keys.F5).key_up(Keys.CONTROL).perform()

    def get_used_memory(self):
        used_memory = get_used_memory(self.driver)
        logger.info("Used memory in the end: %s Mb", used_memory)
        return str(used_memory)

    def _from_list_of_dicts_to_str(self, data: list):
        string = ''
        for i in data:
            for key, value in i.items():
                string += f'_{key}_{value}_'

        return string

    def check_lists_of_dicts(self, list_a: list, list_b: list):
        first_string = self._from_list_of_dicts_to_str(list_a)
        second_string = self._from_list_of_dicts_to_str(list_b)

        return first_string == second_string

    def tear_down(self):
        self.driver.quit()
